/**
 * LinkedList.java
 * 3/30/12
 */

import java.util.NoSuchElementException;
import java.util.Iterator;
import java.lang.Iterable;

/**
 * Implements a linked list of objects.
 * 
 * @author YOUR_NAME_HERE
 * @author AND_HERE
 * @param <T> type of objects in this array list
 */
public class LinkedList<T> implements Iterable<T> {

    /** Number of elements in this linked list. */
    private int size;

    /** Pointer to the first element in the list. */
    private Node first;

    /** Pointer to the last element in the list. */
    private Node last;

    /**
     * A node in a linked list with generic data and pointer to the next
     * node in the list.
     */
    private class Node {

        /** Pointer to the next element in the list. */
        public Node next;

        /** Data stored in this node of the list. */
        public T object;

    }

    /**
     * A front to back iterator for a linked list.
     */
    private class LinkedListIterator implements Iterator<T> {

	/** The node to be returned by the next call to next(). */
	private Node currentNode;
	
	/** Initialize an iterator with the specified first node.
	 *
	 * @param initCurrentNode node to begin iterating from
	 */
	public LinkedListIterator(Node initCurrentNode) {
	    currentNode = initCurrentNode;
	}

	/**
	 * Returns true if the iteration has more elements. (In other
	 * words, returns true if next would return an element rather than
	 * throwing an exception.)
	 *
	 * @return true if the iterator has more elements.
	 */
	public boolean hasNext() {
	    return currentNode != null;
	}

	/**
	 * Returns the next element in the iteration.
	 *
	 * @return the next element in the iteration. 
	 * @throws NoSuchElementException if iteration has no more elements.
	 */
	public T next() {
	    if (currentNode == null) {
		throw new NoSuchElementException("no more elements");
	    }
	    T nextObject = currentNode.object;
	    currentNode = currentNode.next;
	    return nextObject;
	}

	/**
	 * Removing from the linked list is not supported.
	 *
	 * @throws UnsupportedOperationException if the remove operation
	 * is not supported by this Iterator.
	 */
	public void remove() {
	    throw new UnsupportedOperationException("remove is not supported");
	}
    }

    /**
     * Initialize an empty linked list.
     */
    public LinkedList() {
        size = 0;
        first = null;
        last = null;
    }

    /**
     * Add the specified object to the beginning of this linked list.
     * 
     * @param newObject object to add
     */
    public final void addFirst(final T newObject) {
	Node newNode = new Node();
	newNode.object = newObject;
	newNode.next = first;
	if (size == 0) {
	    last = newNode;
	}
	first = newNode;
	size++;
    }

    /**
     * Add the specified object to the end of this linked list.
     * 
     * @param newObject object to add
     */
    public final void addLast(final T newObject) {
	Node newNode = new Node();
	newNode.object = newObject;
	newNode.next = null;
        if (size == 0) {
	    first = newNode;
        } else {
            last.next = newNode;
        }
	last = newNode;
	size++;
    }

    /**
     * Return the first object in this list.
     * 
     * @return the first object in this list
     * @throws NoSuchElementException if the list is empty
     */
    public final T getFirst() {
        if (size == 0) {
            throw new NoSuchElementException("Can not get an element from "
					     + "an empty list.");
        }
        return first.object;
    }

    /**
     * Remove and return the the first object of this linked list.
     * 
     * @return the object that was removed.
     * @throws NoSuchElementException if the list is empty
     */
    public final T removeFirst() {
        if (size == 0) {
            throw new NoSuchElementException("Can not remove an element from "
					     + "an empty list.");
        }
	T removedElement = first.object;
	first = first.next;
	if (size == 1) {
	    last = null;
	}
	size--;
	return removedElement;
    }

    /**
     * Return the number of elements in this linked list.
     * 
     * @return the number of elements in this linked list
     */
    public final int size() {
        return size;
    }

    /**
     * Return a new ArrayListIterator for this array list.
     */
    public Iterator<T> iterator() {
	return new LinkedListIterator(first);
    }

    /**
     * Return a string representing this list.
     * 
     * @return a string representing this list
     */
    public final String toString() {
        StringBuffer string = new StringBuffer();
        Node currentNode = first;
        while (currentNode != null && currentNode.next != null) {
            string.append(currentNode.object.toString() + ", ");
            currentNode = currentNode.next;
        }
        if (currentNode != null) {
            string.append(currentNode.object.toString());
        }
        return string.toString();
    }

}
